@@ -1,6 +1,6 @@
 # Author: Babak Naimi, naimi.b@gmail.com
-# Date (last update):  February 2020
-# Version 4.7
+# Date (last update):  Oct. 2021
+# Version 5.0
 # Licence GPL v3
 
 
@@ -49,6 +49,16 @@ setClass(".methodTemplate",
          )
 )
 #----------
+#--- variable parameters 
+#------- params keeps min, max, mean, sd for each variable
+#------- groups.params is the same as params but stratified based on response variable (e.g., for presence/absence)
+setClass('.variables',
+         representation(
+           names='character',
+           params='data.frame',
+           groups.params="listORnull"
+         )
+)
 
 
 ################################
@@ -56,7 +66,7 @@ setClass(".methodTemplate",
 setClass('sdmFormula',
          representation(
            formula='formula',
-           vars='character',
+           vars='.variables',
            species='characterORnull',
            model.terms='listORnull',
            data.terms='listORnull'
@@ -87,10 +97,9 @@ setClass('.var',
 
 # select function
 setClass('.selectFrame',
-         representation(sets='list',
-                        n='numeric',
-                        stat='character',
-                        keep='characterORnull'
+         representation(vars='character',
+                        n='integer',
+                        stat='character'
          )
 )
 
@@ -112,6 +121,7 @@ setClass('.all.vars',
          )
 )
 
+
 # coordinate columns:
 setClass('.coord.vars',
          representation(
@@ -140,7 +150,8 @@ setClass('.formulaFunction',
          representation(cls='call',
                         name='character',
                         args='character',
-                        getFeature='function'
+                        setFrame='functionORnull',
+                        getFeature='functionORnull'
          )
 )
 
@@ -207,6 +218,7 @@ setRefClass('.formulaFunctions',
 #-----------
 #------------
 
+
 setClass('featuresFrame',
          representation(
            vars='character',
@@ -742,6 +754,7 @@ setClass('.sdmCorSetting',
 setClass('.sdmVariables',
          representation(
            response='character',
+           variables='list',
            distribution='character',
            features.numeric='characterORnull',
            features.factor='characterORnull',
@@ -786,15 +799,20 @@ setRefClass(".workload",
               fit=function(woL=.self,species,models,runs,hasTest,.parMethod=.self$setting@parallelSettings@method,.hostnames=.self$setting@parallelSettings@hosts,.fork=.self$setting@parallelSettings@fork,filename=.self$filename) {
                 .fit(woL=woL,species=species,runs=runs,hasTest = hasTest,.parMethod=.parMethod,.hostnames = .hostnames,.fork = .fork,filename = filename)
               },
-              getSdmVariables=function(sp,nFact) {
+              getSdmVariables=function(sp,nf,nFact) {
                 if (length(.self$sdmVariables) > 0 && !is.null(.self$sdmVariables[[sp]])) .self$sdmVariables[[sp]]
                 else {
                   if (missing(nFact)) {
                     nFact <- .where(is.factor,.self$train[[sp]]$sdmDataFrame)
                     nFact <- names(nFact)[which(nFact)]
                     if (length(nFact) == 0) nFact <- NULL
+                  } else nFact=NULL
+                  
+                  if (missing(nf)) {
+                    nf <- .excludeVector(.self$setting@sdmFormula@vars,nFact)
                   }
-                  .self$sdmVariables[[sp]] <- new('.sdmVariables',response=sp,distribution=.self$setting@distribution[[sp]],features.numeric=.excludeVector(colnames(.self$train[[sp]]$sdmDataFrame),c(sp,nFact)),features.factor=nFact,number.of.records=if (is.null(.self$test)) nrow(.self$train[[sp]]$sdmDataFrame) else c(train=nrow(.self$train[[sp]]$sdmDataFrame),test=nrow(.self$test[[sp]]$sdmDataFrame)))
+                  
+                  .self$sdmVariables[[sp]] <- new('.sdmVariables',response=sp,variables=list(numeric=nf,factors=nFact),distribution=.self$setting@distribution[[sp]],features.numeric=.excludeVector(colnames(.self$train[[sp]]$sdmDataFrame),c(sp,nFact)),features.factor=nFact,number.of.records=if (is.null(.self$test)) nrow(.self$train[[sp]]$sdmDataFrame) else c(train=nrow(.self$train[[sp]]$sdmDataFrame),test=nrow(.self$test[[sp]]$sdmDataFrame)))
                   .self$sdmVariables[[sp]]
                 }
               },
