@@ -1,6 +1,7 @@
 # Author: Babak Naimi, naimi.b@gmail.com
 # Date :  July 2016
-# Version 2.7
+# last update: Oct. 2021
+# Version 2.9
 # Licence GPL v3
 
 
@@ -24,8 +25,7 @@
   ui <- shiny::fluidPage(
     shiny::tags$head(.css),
     
-    shiny::tabsetPanel(position='above',
-                       shiny::tabPanel("Summary", .wellP(shiny::verbatimTextOutput("summary"))),
+    shiny::tabsetPanel(shiny::tabPanel("Summary", .wellP(shiny::verbatimTextOutput("summary"))),
                        shiny::tabPanel("Model Runs Details", .wellP(shiny::tableOutput('run_info'))),
                        shiny::tabPanel("Evaluation",
                                        .wellP(
@@ -114,7 +114,7 @@
                                                    )
                                                    
                                                  )
-                                               ),icon=shiny::icon('line-chart',"fa-2x")
+                                               ),icon=shiny::icon('chart-line',"fa-2x")
                                       ),
                                       shiny::tabPanel('threshold-based',
                                                .wellP(
@@ -143,7 +143,7 @@
                                                    shiny::column(3,shiny::selectInput('wtest7','Which Test Dataset?',wt,selected=wt[length(wt)]))
                                                  )
                                                ),
-                                               .wellP(shiny::tableOutput('thTable2')),icon=shiny::icon('arrows-h',"fa-2x")
+                                               .wellP(shiny::tableOutput('thTable2')),icon=shiny::icon('arrows-alt-h',"fa-2x")
                                       )
                                       
                                       ,id='Statistic'
